"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var hydroWallet_1 = __importDefault(require("./hydroWallet"));
exports.WanWallet = hydroWallet_1.default;
var extensionWallet_1 = __importDefault(require("./extensionWallet"));
exports.ExtensionWallet = extensionWallet_1.default;
var baseWallet_1 = __importDefault(require("./baseWallet"));
exports.BaseWallet = baseWallet_1.default;
var ledger_1 = __importDefault(require("./ledger"));
exports.Ledger = ledger_1.default;
var trezor_1 = __importDefault(require("./trezor"));
exports.Trezor = trezor_1.default;
var lightWallet_1 = __importDefault(require("./lightWallet"));
exports.LightWallet = lightWallet_1.default;
var request_1 = __importDefault(require("request"));
var utils_1 = require("ethers-wan/utils");
var ethers_wan_1 = require("ethers-wan");
var providers_1 = require("ethers-wan/providers");
var web3_1 = __importDefault(require("web3"));
var web3;
var NeedUnlockWalletError = baseWallet_1.default.NeedUnlockWalletError, NotSupportedError = baseWallet_1.default.NotSupportedError;
exports.NeedUnlockWalletError = NeedUnlockWalletError;
exports.NotSupportedError = NotSupportedError;
exports.globalNodeUrl = "https://ropsten.infura.io";
exports.payloadId = function () {
    var datePart = new Date().getTime() * Math.pow(10, 3);
    var extraPart = Math.floor(Math.random() * Math.pow(10, 3));
    var id = datePart + extraPart;
    return id;
};
exports.setGlobalNodeUrl = function (nodeUrl) {
    if (nodeUrl) {
        exports.globalNodeUrl = nodeUrl;
    }
};
exports.truncateAddress = function (address) {
    return address.slice(0, 6) + "..." + address.slice(-4);
};
exports.getNetworkID = function () {
    return new Promise(function (resolve, reject) {
        if (exports.globalNodeUrl.indexOf('ws') === 0) {
            if (!web3) {
                web3 = new web3_1.default(new web3_1.default.providers.WebsocketProvider(exports.globalNodeUrl));
            }
            web3.eth.net.getId(function (err, id) {
                if (err) {
                    web3 = undefined;
                    reject(err);
                }
                else {
                    resolve(id);
                }
            });
        }
        else {
            request_1.default.post(exports.globalNodeUrl, {
                headers: { "content-type": "application/json" },
                form: "{\"jsonrpc\":\"2.0\",\"method\":\"net_version\",\"params\":[],\"id\":" + exports.payloadId() + "}"
            }, function (error, response, data) {
                if (error) {
                    reject(error);
                }
                try {
                    if (data) {
                        var json = JSON.parse(data);
                        resolve(parseInt(json.result, 10));
                    }
                }
                catch (e) {
                    reject(e);
                }
            });
        }
    });
};
exports.sendRawTransaction = function (data) {
    return new Promise(function (resolve, reject) {
        if (exports.globalNodeUrl.indexOf('ws') === 0) {
            if (!web3) {
                web3 = new web3_1.default(new web3_1.default.providers.WebsocketProvider(exports.globalNodeUrl));
            }
            web3.eth.sendSignedTransaction(data, function (err, hash) {
                if (err) {
                    web3 = undefined;
                    reject(err);
                }
                else {
                    resolve(hash);
                }
            });
        }
        else {
            request_1.default.post(exports.globalNodeUrl, {
                headers: { "content-type": "application/json" },
                form: "{\"jsonrpc\":\"2.0\",\"method\":\"eth_sendRawTransaction\",\"params\":[\"" + data + "\"],\"id\":" + exports.payloadId() + "}"
            }, function (error, response, data) {
                if (error) {
                    reject(error);
                }
                try {
                    if (data) {
                        var json = JSON.parse(data);
                        if (json.result) {
                            resolve(json.result);
                        }
                        else if (json.error) {
                            reject(json.error);
                        }
                        else {
                            reject(new Error("Unknown Error"));
                        }
                    }
                }
                catch (e) {
                    reject(e);
                }
            });
        }
    });
};
exports.getBalance = function (address) {
    return new Promise(function (resolve, reject) {
        if (exports.globalNodeUrl.indexOf('ws') === 0) {
            if (!web3) {
                web3 = new web3_1.default(new web3_1.default.providers.WebsocketProvider(exports.globalNodeUrl));
            }
            web3.eth.getBalance(address, function (err, balance) {
                if (err) {
                    web3 = undefined;
                    reject(err);
                }
                else {
                    resolve(new utils_1.BigNumber(balance));
                }
            });
        }
        else {
            request_1.default.post(exports.globalNodeUrl, {
                headers: { "content-type": "application/json" },
                form: "{\"jsonrpc\":\"2.0\",\"method\":\"eth_getBalance\",\"params\":[\"" + address + "\", \"latest\"],\"id\":" + exports.payloadId() + "}"
            }, function (error, response, data) {
                if (error) {
                    reject(error);
                }
                try {
                    if (data) {
                        var json = JSON.parse(data);
                        resolve(new utils_1.BigNumber(json.result));
                    }
                }
                catch (e) {
                    reject(e);
                }
            });
        }
    });
};
exports.getTransactionReceipt = function (txID) {
    return new Promise(function (resolve, reject) {
        if (exports.globalNodeUrl.indexOf('ws') === 0) {
            if (!web3) {
                web3 = new web3_1.default(new web3_1.default.providers.WebsocketProvider(exports.globalNodeUrl));
            }
            web3.eth.getTransactionReceipt(txID, function (err, data) {
                if (err) {
                    web3 = undefined;
                    reject(err);
                }
                else {
                    resolve(data);
                }
            });
        }
        else {
            request_1.default.post(exports.globalNodeUrl, {
                headers: { "content-type": "application/json" },
                form: "{\"jsonrpc\":\"2.0\",\"method\":\"eth_getTransactionReceipt\",\"params\":[\"" + txID + "\"],\"id\":" + exports.payloadId() + "}"
            }, function (error, response, data) {
                if (error) {
                    reject(error);
                }
                try {
                    if (data) {
                        var json = JSON.parse(data);
                        resolve(json.result);
                    }
                }
                catch (e) {
                    reject(e);
                }
            });
        }
    });
};
exports.getEstimateGas = function (params) {
    return new Promise(function (resolve, reject) {
        if (exports.globalNodeUrl.indexOf('ws') === 0) {
            if (!web3) {
                web3 = new web3_1.default(new web3_1.default.providers.WebsocketProvider(exports.globalNodeUrl));
            }
            web3.eth.estimateGas(params, function (err, gas) {
                if (err) {
                    web3 = undefined;
                    reject(err);
                }
                else {
                    resolve(gas);
                }
            });
        }
        else {
            request_1.default.post(exports.globalNodeUrl, {
                headers: { "content-type": "application/json" },
                form: "{\"jsonrpc\":\"2.0\",\"method\":\"eth_estimateGas\",\"params\":[" + JSON.stringify(params) + "],\"id\":" + exports.payloadId() + "}"
            }, function (error, response, data) {
                if (error) {
                    reject(error);
                }
                try {
                    if (data) {
                        var json = JSON.parse(data);
                        resolve(json.result);
                    }
                }
                catch (e) {
                    reject(e);
                }
            });
        }
    });
};
exports.getTransactionCount = function (address, status) {
    if (status === void 0) { status = "latest"; }
    return new Promise(function (resolve, reject) {
        if (exports.globalNodeUrl.indexOf('ws') === 0) {
            if (!web3) {
                web3 = new web3_1.default(new web3_1.default.providers.WebsocketProvider(exports.globalNodeUrl));
            }
            web3.eth.getTransactionCount(address, function (err, data) {
                if (err) {
                    web3 = undefined;
                    reject(err);
                }
                else {
                    resolve(data);
                }
            });
        }
        else {
            request_1.default.post(exports.globalNodeUrl, {
                headers: { "content-type": "application/json" },
                form: "{\"jsonrpc\":\"2.0\",\"method\":\"eth_getTransactionCount\",\"params\":[\"" + address + "\",\"" + status + "\"],\"id\":" + exports.payloadId() + "}"
            }, function (error, response, data) {
                if (error) {
                    reject(error);
                }
                try {
                    if (data) {
                        var json = JSON.parse(data);
                        resolve(Number(json.result));
                    }
                }
                catch (e) {
                    reject(e);
                }
            });
        }
    });
};
exports.getContract = function (contractAddress, abi) {
    var provider = new providers_1.JsonRpcProvider(exports.globalNodeUrl);
    return new ethers_wan_1.Contract(contractAddress, abi, provider);
};
exports.defaultWalletTypes = [extensionWallet_1.default.TYPE, hydroWallet_1.default.TYPE, ledger_1.default.TYPE, trezor_1.default.TYPE, lightWallet_1.default.TYPE];
